#ifndef _audioformat__h_
#define _audioformat__h_

#include <windows.h>
#include <cstdint>

/**
 * Represents the various audio encodings we intend to support.
 */
enum class AudioEncoding
{
    FLOATING_POINT,
    PCM_8,
    PCM_16,
    PCM_24,
    PCM_24_IN_32,
    PCM_32,
    UNSUPPORTED,
};

/**
 * Convenience wrapper for WAVEFORMATEX.
 */
class AudioFormat
{
public:

    // Takes ownership of the WAVEFORMATEX, which should have
    // been allocated using CoTaskMemAlloc.
    explicit AudioFormat(WAVEFORMATEX *pWaveFormat);        

    ~AudioFormat();

    inline AudioEncoding getEncoding() const { return m_encoding; }
    inline int getNumChannels() const { return m_pWaveFormat->nChannels; }
        
    inline uint32_t getSampleRateInHz() const 
    { 
        return m_pWaveFormat->nSamplesPerSec; 
    }
        
    inline int getBytesPerSample() const 
    { 
        return m_pWaveFormat->wBitsPerSample / 8; 
    }
        
    inline int getFrameSizeInBytes() const 
    { 
        return m_pWaveFormat->nBlockAlign; 
    }

    // Returns the underlying WAVEFORMATEX.
    const WAVEFORMATEX * getWaveFormat() const { return m_pWaveFormat; }

private:

    // Copy constructor. Not implemented.
    AudioFormat(const AudioFormat &) = delete;
    // Assignment operator. Not implemented.
    const AudioFormat & operator=(const AudioFormat &) = delete;

    static AudioEncoding determineAudioEncoding(const WAVEFORMATEX &ex);
        
    WAVEFORMATEX *m_pWaveFormat;

    // Cached for performance.
    AudioEncoding m_encoding;
};

#endif
