#include "CaptureThread.h"
#include "CaptureStream.h"
#include "DataBuffer.h"

#include <stdexcept>
#include <avrt.h>
#include <iostream>

CaptureThread::CaptureThread(CaptureStream &stream, DataBufferWriter &buffer) :
    m_stream(stream), m_buffer(buffer), m_hThread(NULL), 
    m_stopThread(true), m_isFinished(true)
{
}

void
CaptureThread::start()
{
    if (m_hThread != NULL)
        throw std::runtime_error("Render thread already running.");

    m_stopThread = false;
    m_isFinished = false;
    m_hThread = CreateThread(NULL, 0, threadProc, this, 0, NULL);
}

void
CaptureThread::stop()
{
    if (m_hThread == NULL)
        return; // Nothing to do.

    m_stopThread = true;
    WaitForSingleObject(m_hThread, INFINITE);
    CloseHandle(m_hThread);
    m_hThread = NULL;
}

DWORD
CaptureThread::threadProc(LPVOID param)
{
    HRESULT hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);

    // Increasing thread priority.
    DWORD mmTaskIndex = 0;
    HANDLE mmTaskHandle = AvSetMmThreadCharacteristicsA("Audio", &mmTaskIndex);

    CaptureThread *pThread = reinterpret_cast<CaptureThread *>(param);

    CaptureStream &stream = pThread->m_stream;
    DataBufferWriter &buffer = pThread->m_buffer;
    bool &stopThread = pThread->m_stopThread;

    try
    {
        stream.startStream();
        while (!stopThread && !buffer.isBufferFull())
        {
            stream.waitOnBuffer();
            stream.captureNextFrames(buffer);
        }
    }
    catch (const std::runtime_error &e)
    {
        std::cerr << "Error: " << e.what() << std::endl;
    }

    stream.stopStream();

    // Reverting our thread priority changes.
    if (mmTaskHandle != NULL)
        AvRevertMmThreadCharacteristics(mmTaskHandle);

    pThread->m_isFinished = true;

    CoUninitialize();

    return 0;
}
