#include "DataBuffer.h"
#include "AudioFormat.h"

#include <stdexcept>
#include <memory>
#include <algorithm>

uint32_t 
DataBufferWriter::writeBytes(unsigned char *pBuffer, uint32_t bufferLen)
{
    if (m_pBuffer == nullptr)
        return 0;

    if (m_bufferUsed >= m_bufferSize)
        return 0;

    uint32_t to_write = (std::min)(bufferLen, (m_bufferSize - m_bufferUsed));
    memcpy(m_pBuffer + m_bufferUsed, pBuffer, to_write);
    m_bufferUsed += to_write;
    return to_write;
}

uint32_t
DataBufferWriter::writeValue(unsigned char byte, uint32_t count)
{
    if (m_pBuffer == nullptr)
        return 0;

    if (m_bufferUsed >= m_bufferSize)
        return 0;

    uint32_t to_write = (std::min)(count, (m_bufferSize - m_bufferUsed));
    memset(m_pBuffer + m_bufferUsed, byte, to_write);
    m_bufferUsed += to_write;
    return to_write;
}

uint32_t
DataBufferReader::readBytes(unsigned char *pBuffer, uint32_t bufferLen)
{
    if (m_pBuffer == nullptr)
        return 0;

    if (m_bytesRead >= m_bufferSize)
        return 0; // Everything has been read.

    uint32_t to_read = (std::min)(bufferLen, (m_bufferSize - m_bytesRead));
    memcpy(pBuffer, m_pBuffer + m_bytesRead, to_read);
    m_bytesRead += to_read;
    return to_read;
}
