#include "KeyMonitor.h"

KeyMonitor::KeyMonitor()
{    
    m_hConsole = GetStdHandle(STD_INPUT_HANDLE);
}

KeyMonitor::KeyPress
KeyMonitor::getKeyPress(bool waitForKeyPress) const
{
    INPUT_RECORD inputRecord = { 0 };
    DWORD eventsRead = 0;

    if (!waitForKeyPress)
    {
        PeekConsoleInput(m_hConsole, &inputRecord, 1, &eventsRead);
        if (eventsRead == 0)
        {
            // No keys pressed
            return KeyPress::None;
        }
    }
    
    // There was an input event. We need to read it so it's removed
    // from the input buffer. 
    ReadConsoleInput(m_hConsole, &inputRecord, 1, &eventsRead);
    if (inputRecord.EventType != KEY_EVENT)
    {
        // No keys pressed. Some other input event occurred.
        return KeyPress::None;
    }

    // For every key press, there will be two events. One for keydown
    // and one for key up. We'll only concern ourselves with keydown.
    if (!inputRecord.Event.KeyEvent.bKeyDown)
    {
        return KeyPress::None;
    }

    switch (inputRecord.Event.KeyEvent.wVirtualKeyCode)
    {
        case VK_RIGHT : return KeyPress::Right;
        case VK_LEFT  : return KeyPress::Left;
        case VK_UP    : return KeyPress::Up;
        case VK_DOWN  : return KeyPress::Down;
        case 0x51     : return KeyPress::Q;
        default       : return KeyPress::None; // Something we're not interested in.
    }
}