#ifndef _capturestream__h_
#define _capturestream__h_

#include "ComTypes.h"
#include "AudioFormat.h"

#include <memory>

class DataBufferWriter;

/**
 * Abstracts some of the complexity behind creating and using a 
 * capture audio stream.
 */
class CaptureStream
{
public:

    // Constructor.
    CaptureStream(IMMDevicePtr &spDevice);
    // Destructor.
    ~CaptureStream();

    // Returns the current rendering audio format.
    std::shared_ptr<AudioFormat> getAudioFormat() const { return m_spAudioFormat; }

    // Starts the audio stream.
    void startStream() { m_spAudioClient->Start(); }
    // Stops the audio stream.
    void stopStream() { m_spAudioClient->Stop(); }

    // Performs a wait appropriate for our buffering strategy. In this case,
    // it's an event. Returns false if a timeout occurred.
    bool waitOnBuffer();

    // Captures data into the given buffer.
    void captureNextFrames(DataBufferWriter &dataBuffer);

private:

    // COM interfaces
    IAudioClientPtr m_spAudioClient;
    IAudioCaptureClientPtr m_spCaptureClient;

    // The audio format.
    std::shared_ptr<AudioFormat> m_spAudioFormat;

    // Event handle
    HANDLE m_hEvent;
};

#endif
