#ifndef _renderstream__h_
#define _renderstream__h_

#include "ComTypes.h"
#include "AudioFormat.h"

#include <memory>

class DataBufferReader;

/**
 * Abstracts some of the complexity behind creating and using a 
 * rendering audio stream.
 */
class RenderStream
{
public:

    // Constructor.
    RenderStream(IMMDevicePtr &spDevice, 
        const std::shared_ptr<AudioFormat> &spAudioFormat);
    // Destructor.
    ~RenderStream();

    // Returns the current rendering audio format.
    std::shared_ptr<AudioFormat> getAudioFormat() const { return m_spAudioFormat; }

    // Starts the audio stream.
    void startStream() { m_spAudioClient->Start(); }
    // Stops the audio stream.
    void stopStream() { m_spAudioClient->Stop(); }

    // Performs a wait appropriate for our buffering strategy. In this case,
    // it's an event. Returns false if a timeout occurred.
    bool waitOnBuffer();

    // Renders data from the given data buffer.
    void renderNextFrames(DataBufferReader &bufferReader);

private:

    // COM interfaces
    IAudioClientPtr m_spAudioClient;
    IAudioRenderClientPtr m_spRenderClient;

    // We cache this so we don't have to keep looking it up.
    UINT32 m_bufferSizeInFrames;

    // The audio format.
    std::shared_ptr<AudioFormat> m_spAudioFormat;

    // Event handle
    HANDLE m_hEvent;
};

#endif
