#include "AudioEndpointEnumerator.h"

#include <stdexcept>

AudioEndpointEnumerator::AudioEndpointEnumerator()
{
    // Attemping to obtain an instance of the IMMDeviceEnumerator. 
    if (FAILED(CoCreateInstance(__uuidof(MMDeviceEnumerator), NULL, CLSCTX_ALL,
        __uuidof(IMMDeviceEnumerator), (void **)&m_spDeviceEnumerator)))
    {
        throw std::runtime_error(
            "Failed to create an instance of the device enumerator.");
    }
}

size_t
AudioEndpointEnumerator::enumerateAudioDevices(std::vector<IMMDevicePtr> &foundDevices)
{
    // Enumerating all audio endpoints.
    IMMDeviceCollectionPtr spDeviceCollection;
    if (FAILED(m_spDeviceEnumerator->EnumAudioEndpoints(eAll, DEVICE_STATE_ACTIVE,
        &spDeviceCollection)))
    {
        throw std::runtime_error("No endpoints found.");
    }

    // Iterating over all of the audio endpoints and adding them foundDevices.
    UINT itemCount = 0;
    spDeviceCollection->GetCount(&itemCount);

    for (UINT i = 0; i < itemCount; ++i)
    {
        IMMDevicePtr spDevice;
        if (SUCCEEDED(spDeviceCollection->Item(i, &spDevice)))
            foundDevices.push_back(spDevice);
    }

    return foundDevices.size();
}
