#include "AudioDevices.h"

#include <stdexcept>

IMMDevicePtr 
AudioDevices::getDefaultRenderDevice()
{
    IMMDeviceEnumeratorPtr spEnumerator = getDeviceEnumerator();
    IMMDevicePtr spDevice;
    if (FAILED(spEnumerator->GetDefaultAudioEndpoint(eRender, eMultimedia, &spDevice)))
    {
        throw std::runtime_error("Failed to acquire default audio render device.");
    }
    return spDevice;
}

IMMDevicePtr 
AudioDevices::getDefaultCaptureDevice()
{
    IMMDeviceEnumeratorPtr spEnumerator = getDeviceEnumerator();
    IMMDevicePtr spDevice;
    if (FAILED(spEnumerator->GetDefaultAudioEndpoint(eCapture, eMultimedia, &spDevice)))
    {
        throw std::runtime_error("Failed to acquire default audio capture endpoint.");
    }
    return spDevice;
}

IMMDeviceEnumeratorPtr 
AudioDevices::getDeviceEnumerator()
{
    IMMDeviceEnumeratorPtr spEnumerator;
    if (FAILED(CoCreateInstance(__uuidof(MMDeviceEnumerator), NULL, CLSCTX_ALL,
        __uuidof(IMMDeviceEnumerator), (void **)&spEnumerator)))
    {
        throw std::runtime_error("Failed to acquire device enumerator.");
    }
    return spEnumerator;
}
