#ifndef _referencetimeutils__h_
#define _referencetimeutils__h_

#include <ks.h>
#include <ksmedia.h>
#include <cstdint>

/**
 * Utility class for converting between REFERENCE_TIMEs and other time
 * units.
 *
 * REFERENCE_TIMES are in 100 nanosecond units. The methods below are
 * intended to ease the mental yoga required to make use of regular 
 * time units.
 */
class ReferenceTimeUtils
{
public:
    
    static REFERENCE_TIME fromMilliseconds(double milliseconds) 
    {
        // 1000000 nanoseconds in a millisecond, divided by 100 for REFERENCE_TIME.
        return static_cast<REFERENCE_TIME>(milliseconds * 10000.0);
    }
           
    static REFERENCE_TIME fromSeconds(double seconds)
    {
        // 1000000000 nanoseconds in a second, divided by 100 for REFERENCE_TIME.
        return static_cast<REFERENCE_TIME>(seconds * 10000000.0);
    }
        
    static double toMilliseconds(REFERENCE_TIME refTimes)
    {
        // 1000000 nanoseconds in a millisecond, divided by 100 for REFERENCE_TIME.
        return static_cast<double>(refTimes / 10000.0);
    }
    
    static double toSeconds(REFERENCE_TIME refTimes)
    {
        // 1000000000 nanoseconds in a second, divided by 100 for REFERENCE_TIME. 
        return static_cast<double>(refTimes / 10000000.0);
    }

private:

    // Constructor. NOT IMPLEMENTED.
    ReferenceTimeUtils() = delete;
};

#endif
