#include "RenderThread.h"
#include "RenderStream.h"
#include "SineWaveGenerator.h"

#include <stdexcept>
#include <avrt.h>
#include <iostream>

RenderThread::RenderThread(RenderStream &stream, SineWaveGenerator &sampleSource) :
    m_stream(stream), m_sampleSource(sampleSource), m_hThread(NULL), m_stopThread(true)
{
}

void
RenderThread::start()
{
    if (m_hThread != NULL)
        throw std::runtime_error("Render thread already running.");

    m_stopThread = false;
    m_hThread = CreateThread(NULL, 0, threadProc, this, 0, NULL);
}

void
RenderThread::stop()
{
    if (m_hThread == NULL)
        return; // Nothing to do.

    m_stopThread = true;
    WaitForSingleObject(m_hThread, INFINITE);
    CloseHandle(m_hThread);
    m_hThread = NULL;
}

DWORD
RenderThread::threadProc(LPVOID param)
{
    HRESULT hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);

    // Increasing thread priority.
    DWORD mmTaskIndex = 0;
    HANDLE mmTaskHandle = AvSetMmThreadCharacteristicsA("Audio", &mmTaskIndex);

    RenderThread *pThread = reinterpret_cast<RenderThread *>(param);

    RenderStream &stream = pThread->m_stream;
    SineWaveGenerator &sampleSource = pThread->m_sampleSource;
    bool &stopThread = pThread->m_stopThread;

    try
    {
        // Preload the buffer. We must do this before starting the audio stream.
        stream.renderNextFrames(sampleSource);
        stream.startStream();
        while (!stopThread)
        {
            if (stream.waitOnBuffer())
                stream.renderNextFrames(sampleSource);
        }
    }
    catch (const std::runtime_error &e)
    {
        std::cerr << "Error: " << e.what() << std::endl;
    }

    stream.stopStream();

    // Reverting our thread priority changes.
    if (mmTaskHandle != NULL)
        AvRevertMmThreadCharacteristics(mmTaskHandle);

    CoUninitialize();

    return 0;
}
