#ifndef _sinewavegenerator__h_
#define _sinewavegenerator__h_

#include <cstdint>
#include <atomic>

/**
 * Simple sine wave generator.
 */
class SineWaveGenerator
{

public:

    // Constructor.
    SineWaveGenerator();

    // Sets the frequency of the sine wave.
    void setFrequencyInHz(double frequencyInHz) 
    { 
        if (frequencyInHz < 0.0)
            frequencyInHz = 0.0;
        m_frequencyInHz = frequencyInHz; 
    }

    // Returns the current frequency.
    double getFrequencyInHz() const { return m_frequencyInHz; }

    // Returns the next sample of audio data.
    virtual double nextSample(int64_t sampleRateInHz);

private:    

    // Eg. 440 (A4), 493.88 (B4), 523.5 (C5), etc.
    std::atomic<double> m_frequencyInHz; 

    // State that allows us to keep track of where we are in the
    // sine wave generation.
    double m_phase;    
};

#endif
