#ifndef _renderstream__h_
#define _renderstream__h_

#include "ComTypes.h"
#include "AudioFormat.h"

#include <atomic>
#include <memory>

class SineWaveGenerator;

/**
 * Represents a rendering audio stream. Abstracts some of the
 * complexity behind creating and using a rendering audio stream.
 */
class RenderStream
{
public:

    // Constructor.
    RenderStream(IMMDevicePtr &spDevice);

    // Sets the current volume, should be a value between 0 and 1.
    void setVolume(double volume);
    // Gets the current volume, a value between 0 and 1.
    double getVolume() const { return m_volume; }

    // Returns the current rendering audio format.
    std::shared_ptr<AudioFormat> getAudioFormat() const { return m_spAudioFormat; }

    // Starts the audio stream.
    void startStream() { m_spAudioClient->Start(); }
    // Stops the audio stream.
    void stopStream() { m_spAudioClient->Stop(); }

    // Performs a wait appropriate for our buffering strategy. In this case,
    // it's a timer. Returns false if a timeout occurred.
    bool waitOnBuffer();

    // Renders data from the given SineWaveGenerator.
    void renderNextFrames(SineWaveGenerator &sampleSource);

private:

    // Fills the audio buffer with data from the given audio source.
    void fillBuffer(SineWaveGenerator &sampleSource, BYTE *pBuffer,
        UINT32 bufferSizeInFrames);

    // COM interfaces
    IAudioClientPtr m_spAudioClient;
    IAudioRenderClientPtr m_spRenderClient;

    // We cache this so we don't have to keep looking it up.
    UINT32 m_bufferSizeInFrames;

    // User configurable volume.
    std::atomic<double> m_volume; 

    // The audio format.
    std::shared_ptr<AudioFormat> m_spAudioFormat;

    // How long we should sleep, waiting for the buffer to be free.
    DWORD m_sleepTimeInMs;
};

#endif
