#include "SineWaveGenerator.h"

#include <algorithm>

SineWaveGenerator::SineWaveGenerator() : 
    m_frequencyInHz(400.0), // Some reasonable default
    m_phase(0.0)
{
}

double 
SineWaveGenerator::nextSample(int64_t sampleRateInHz)
{
    static const double TWOPI_RADIAN = 2.0 * 3.14159;

    // Sample rate should be > 0. Also, returning prevents a divide by 
    // zero below. 
    if (sampleRateInHz == 0.0)
        return 0.0;

    // If frequency never changed, we could pre-compute these two items
    // as an optimization.
    const double cyclesPerSample = m_frequencyInHz / sampleRateInHz;
    const double phaseDelta = cyclesPerSample * TWOPI_RADIAN;

    double sample = std::sin(m_phase);

    // Bumping our phase forward.
    m_phase += phaseDelta;

    // This will prevent overflow, which can cause weird audio glitches.
    if (m_phase > TWOPI_RADIAN)
        m_phase -= TWOPI_RADIAN;

    return sample;
}
